@@ -43,8 +43,8 @@ int conv(int num_msg, const struct pam_message **msg,
     Panel* panel = *static_cast<Panel**>(appdata_ptr);
     int result = PAM_SUCCESS;
     for (int i=0; i<num_msg; i++){
-        resp[i]->resp=0;
-        resp[i]->resp_retcode=0;
+        (*resp)[i].resp=0;
+        (*resp)[i].resp_retcode=0;
         switch(msg[i]->msg_style){
             case PAM_PROMPT_ECHO_ON:
                 // We assume PAM is asking for the username
@@ -53,13 +53,13 @@ int conv(int num_msg, const struct pam_message **msg,
                     case Panel::Suspend:
                     case Panel::Halt:
                     case Panel::Reboot:
-                        resp[i]->resp=strdup("root");
+                        (*resp)[i].resp=strdup("root");
                         break;
 
                     case Panel::Console:
                     case Panel::Exit:
                     case Panel::Login:
-                        resp[i]->resp=strdup(panel->GetName().c_str());
+                        (*resp)[i].resp=strdup(panel->GetName().c_str());
                         break;
                 }
                 break;
@@ -75,7 +75,7 @@ int conv(int num_msg, const struct pam_message **msg,
 
                     default:
                         panel->EventHandler(Panel::Get_Passwd);
-                        resp[i]->resp=strdup(panel->GetPasswd().c_str());
+                        (*resp)[i].resp=strdup(panel->GetPasswd().c_str());
                         break;
                 }
                 break;
@@ -84,16 +84,16 @@ int conv(int num_msg, const struct pam_message **msg,
             case PAM_TEXT_INFO:
                 // We simply write these to the log
                 // TODO: Maybe we should simply ignore them
-                cerr << APPNAME << ": " << msg[i]->msg << endl;
+                logStream << APPNAME << ": " << msg[i]->msg << endl;
                 break;
         }
         if (result!=PAM_SUCCESS) break;
     }
     if (result!=PAM_SUCCESS){
         for (int i=0; i<num_msg; i++){
-            if (resp[i]->resp==0) continue;
-            free(resp[i]->resp);
-            resp[i]->resp=0;
+            if ((*resp)[i].resp==0) continue;
+            free((*resp)[i].resp);
+            (*resp)[i].resp=0;
         };
         free(*resp);
         *resp=0;
@@ -104,29 +104,21 @@ int conv(int num_msg, const struct pam_message **msg,
 
 extern App* LoginApp;
 
-void CatchSignal(int sig) {
-    cerr << APPNAME << ": unexpected signal " << sig << endl;
+int xioerror(Display *disp) {
+	LoginApp->RestartServer();
+    return 0;
+}
 
-    if (LoginApp->serverStarted)
+void CatchSignal(int sig) {
+    logStream << APPNAME << ": unexpected signal " << sig << endl;
+
+    if (LoginApp->isServerStarted())
         LoginApp->StopServer();
 
     LoginApp->RemoveLock();
     exit(ERR_EXIT);
 }
 
-
-void AlarmSignal(int sig) {
-    int pid = LoginApp->GetServerPID();
-    if(waitpid(pid, NULL, WNOHANG) == pid) {
-        LoginApp->StopServer();
-        LoginApp->RemoveLock();
-        exit(OK_EXIT);
-    }
-    signal(sig, AlarmSignal);
-    alarm(2);
-}
-
-
 void User1Signal(int sig) {
     signal(sig, User1Signal);
 }
@@ -159,7 +151,7 @@ App::App(int argc, char** argv)
             testtheme = optarg;
             testing = true;
             if (testtheme == NULL) {
-                cerr << "The -p option requires an argument" << endl;
+                logStream << "The -p option requires an argument" << endl;
                 exit(ERR_EXIT);
             }
             break;
@@ -175,9 +167,9 @@ App::App(int argc, char** argv)
             exit(OK_EXIT);
             break;
         case '?':    // Illegal
-            cerr << endl;
+            logStream << endl;
         case 'h':   // Help
-            cerr << "usage:  " << APPNAME << " [option ...]" << endl
+            logStream << "usage:  " << APPNAME << " [option ...]" << endl
             << "options:" << endl
             << "    -d: daemon mode" << endl
             << "    -nodaemon: no-daemon mode" << endl
@@ -189,7 +181,7 @@ App::App(int argc, char** argv)
     }
 #ifndef XNEST_DEBUG 
     if (getuid() != 0 && !testing) {
-        cerr << APPNAME << ": only root can run this program" << endl;
+        logStream << APPNAME << ": only root can run this program" << endl;
         exit(ERR_EXIT);
     }
 #endif /* XNEST_DEBUG */
@@ -236,11 +228,9 @@ void App::Run() {
         pam.start("slim");
         pam.set_item(PAM::Authenticator::TTY, DisplayName);
         pam.set_item(PAM::Authenticator::Requestor, "root");
-        pam.set_item(PAM::Authenticator::Host, "localhost");
-
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
         exit(ERR_EXIT);
     };
 #endif
@@ -251,11 +241,11 @@ void App::Run() {
         themefile = themedir + THEMESFILE;
         if (!cfg->readConf(themefile)) {
             if (themeName == "default") {
-                cerr << APPNAME << ": Failed to open default theme file "
+                logStream << APPNAME << ": Failed to open default theme file "
                      << themefile << endl;
                 exit(ERR_EXIT);
             } else {
-                cerr << APPNAME << ": Invalid theme in config: "
+                logStream << APPNAME << ": Invalid theme in config: "
                      << themeName << endl;
                 themeName = "default";
             }
@@ -277,34 +267,34 @@ void App::Run() {
         signal(SIGHUP, CatchSignal);
         signal(SIGPIPE, CatchSignal);
         signal(SIGUSR1, User1Signal);
-        signal(SIGALRM, AlarmSignal);
 
 #ifndef XNEST_DEBUG
-        OpenLog();
-        
         if (!force_nodaemon && cfg->getOption("daemon") == "yes") {
             daemonmode = true;
         }
 
         // Daemonize
         if (daemonmode) {
-            if (daemon(0, 1) == -1) {
-                cerr << APPNAME << ": " << strerror(errno) << endl;
+            if (daemon(0, 0) == -1) {
+                logStream << APPNAME << ": " << strerror(errno) << endl;
                 exit(ERR_EXIT);
             }
-            UpdatePid();
         }
 
+        OpenLog();
+
+        if (daemonmode)
+            UpdatePid();
+
         CreateServerAuth();
         StartServer();
-        alarm(2);
 #endif
 
     }
 
     // Open display
     if((Dpy = XOpenDisplay(DisplayName)) == 0) {
-        cerr << APPNAME << ": could not open display '"
+        logStream << APPNAME << ": could not open display '"
              << DisplayName << "'" << endl;
         if (!testing) StopServer();
         exit(ERR_EXIT);
@@ -342,6 +332,15 @@ void App::Run() {
             Login();
         }
     }
+
+    // Set NumLock
+    string numlock = cfg->getOption("numlock");
+    if (numlock == "on") {
+        NumLock::setOn(Dpy);
+    } else if (numlock == "off") {
+        NumLock::setOff(Dpy);
+    }
+    
     // Start looping
     int panelclosed = 1;
     Panel::ActionType Action;
@@ -426,11 +425,11 @@ bool App::AuthenticateUser(bool focuspass){
             default:
                 break;
         };
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
         return false;
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
         exit(ERR_EXIT);
     };
     return true;
@@ -442,7 +441,7 @@ bool App::AuthenticateUser(bool focuspass){
         switch(LoginPanel->getAction()){
             case Panel::Exit:
             case Panel::Console:
-                cerr << APPNAME << ": Got a special command (" << LoginPanel->GetName() << ")" << endl;
+                logStream << APPNAME << ": Got a special command (" << LoginPanel->GetName() << ")" << endl;
                 return true; // <--- This is simply fake!
             default:
                 break;
@@ -519,11 +518,11 @@ void App::Login() {
     }
     catch(PAM::Cred_Exception& e){
         // Credentials couldn't be established
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
         return;
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
         exit(ERR_EXIT);
     };
 #else
@@ -551,6 +550,7 @@ void App::Login() {
     try{
         if(term) pam.setenv("TERM", term);
         pam.setenv("HOME", pw->pw_dir);
+        pam.setenv("PWD", pw->pw_dir);
         pam.setenv("SHELL", pw->pw_shell);
         pam.setenv("USER", pw->pw_name);
         pam.setenv("LOGNAME", pw->pw_name);
@@ -560,7 +560,18 @@ void App::Login() {
         pam.setenv("XAUTHORITY", xauthority.c_str());
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
+        exit(ERR_EXIT);
+    }
+#endif
+
+#ifdef USE_CONSOLEKIT
+    // Setup the ConsoleKit session
+    try {
+        ck.open_session(DisplayName, pw->pw_uid);
+    }
+    catch(Ck::Exception &e) {
+        logStream << APPNAME << ": " << e << endl;
         exit(ERR_EXIT);
     }
 #endif
@@ -572,13 +583,35 @@ void App::Login() {
         // Get a copy of the environment and close the child's copy
         // of the PAM-handle.
         char** child_env = pam.getenvlist();
+
+# ifdef USE_CONSOLEKIT
+        char** old_env = child_env;
+
+        // Grow the copy of the environment for the session cookie
+        int n;
+        for(n = 0; child_env[n] != NULL ; n++);
+
+        n++;
+
+        child_env = static_cast<char**>(malloc(sizeof(char*)*n));
+        memcpy(child_env, old_env, sizeof(char*)*n);
+        child_env[n - 1] = StrConcat("XDG_SESSION_COOKIE=", ck.get_xdg_session_cookie());
+        child_env[n] = NULL;
+# endif /* USE_CONSOLEKIT */
+
         pam.end();
 #else
-        const int Num_Of_Variables = 10; // Number of env. variables + 1
+
+# ifdef USE_CONSOLEKIT
+        const int Num_Of_Variables = 12; // Number of env. variables + 1
+# else
+        const int Num_Of_Variables = 11; // Number of env. variables + 1
+# endif /* USE_CONSOLEKIT */
         char** child_env = static_cast<char**>(malloc(sizeof(char*)*Num_Of_Variables));
         int n = 0;
         if(term) child_env[n++]=StrConcat("TERM=", term);
         child_env[n++]=StrConcat("HOME=", pw->pw_dir);
+        child_env[n++]=StrConcat("PWD=", pw->pw_dir);
         child_env[n++]=StrConcat("SHELL=", pw->pw_shell);
         child_env[n++]=StrConcat("USER=", pw->pw_name);
         child_env[n++]=StrConcat("LOGNAME=", pw->pw_name);
@@ -586,7 +619,11 @@ void App::Login() {
         child_env[n++]=StrConcat("DISPLAY=", DisplayName);
         child_env[n++]=StrConcat("MAIL=", maildir.c_str());
         child_env[n++]=StrConcat("XAUTHORITY=", xauthority.c_str());
+# ifdef USE_CONSOLEKIT
+        child_env[n++]=StrConcat("XDG_SESSION_COOKIE=", ck.get_xdg_session_cookie());
+# endif /* USE_CONSOLEKIT */
         child_env[n++]=0;
+
 #endif
 
         // Login process starts here
@@ -613,6 +650,8 @@ void App::Login() {
     int status;
     while (wpid != pid) {
         wpid = wait(&status);
+		if (wpid == ServerPID)
+			xioerror(Dpy);	// Server died, simulate IO error
     }
     if (WIFEXITED(status) && WEXITSTATUS(status)) {
         LoginPanel->Message("Failed to execute login command");
@@ -625,12 +664,21 @@ void App::Login() {
         }
     }
 
+#ifdef USE_CONSOLEKIT
+    try {
+        ck.close_session();
+    }
+    catch(Ck::Exception &e) {
+        logStream << APPNAME << ": " << e << endl;
+    };
+#endif
+
 #ifdef USE_PAM
     try{
         pam.close_session();
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
     };
 #endif
 
@@ -658,15 +706,12 @@ void App::Login() {
 
 
 void App::Reboot() {
-    // Stop alarm clock
-    alarm(0);
-
 #ifdef USE_PAM
     try{
         pam.end();
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
     };
 #endif
 
@@ -683,15 +728,12 @@ void App::Reboot() {
 
 
 void App::Halt() {
-    // Stop alarm clock
-    alarm(0);
-
 #ifdef USE_PAM
     try{
         pam.end();
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
     };
 #endif
 
@@ -735,7 +777,7 @@ void App::Exit() {
         pam.end();
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
     };
 #endif
 
@@ -765,12 +807,13 @@ void App::RestartServer() {
         pam.end();
     }
     catch(PAM::Exception& e){
-        cerr << APPNAME << ": " << e << endl;
+        logStream << APPNAME << ": " << e << endl;
     };
 #endif
 
     StopServer(); 
     RemoveLock();
+	while (waitpid(-1, NULL, WNOHANG) > 0); // Collects all dead childrens
     Run();
 } 
 
@@ -817,9 +860,9 @@ int App::ServerTimeout(int timeout, char* text) {
             break;
         if(timeout) {
             if(i == 0 && text != lasttext)
-                cerr << endl << APPNAME << ": waiting for " << text;
+                logStream << endl << APPNAME << ": waiting for " << text;
             else
-                cerr << ".";
+                logStream << ".";
         }
         if(timeout)
             sleep(1);
@@ -828,7 +871,7 @@ int App::ServerTimeout(int timeout, char* text) {
     }
 
     if(i > 0)
-        cerr << endl;
+        logStream << endl;
     lasttext = text;
 
     return (ServerPID != pidfound);
@@ -841,6 +884,7 @@ int App::WaitForServer() {
 
     for(cycles = 0; cycles < ncycles; cycles++) {
         if((Dpy = XOpenDisplay(DisplayName))) {
+            XSetIOErrorHandler(xioerror);
             return 1;
         } else {
             if(!ServerTimeout(1, (char *) "X server to begin accepting connections"))
@@ -848,14 +892,14 @@ int App::WaitForServer() {
         }
     }
 
-    cerr << "Giving up." << endl;
+    logStream << "Giving up." << endl;
 
     return 0;
 }
 
 
 int App::StartServer() {
-    ServerPID = vfork();
+    ServerPID = fork();
 
     static const int MAX_XSERVER_ARGS = 256;
     static char* server[MAX_XSERVER_ARGS+2] = { NULL };
@@ -912,7 +956,7 @@ int App::StartServer() {
 
 
         execvp(server[0], server);
-        cerr << APPNAME << ": X server could not be started" << endl;
+        logStream << APPNAME << ": X server could not be started" << endl;
         exit(ERR_EXIT);
         break;
 
@@ -925,13 +969,10 @@ int App::StartServer() {
             ServerPID = -1;
             break;
         }
-        alarm(15);
-        pause();
-        alarm(0);
 
         // Wait for server to start up
         if(WaitForServer() == 0) {
-            cerr << APPNAME << ": unable to connect to X server" << endl;
+            logStream << APPNAME << ": unable to connect to X server" << endl;
             StopServer();
             ServerPID = -1;
             exit(ERR_EXIT);
@@ -939,13 +980,6 @@ int App::StartServer() {
         break;
     }
 
-    string numlock = cfg->getOption("numlock");
-    if (numlock == "on") {
-        NumLock::setOn();
-    } else if (numlock == "off") {
-        NumLock::setOff();
-    }
-    
     delete args;
 
     serverStarted = true;
@@ -956,21 +990,18 @@ int App::StartServer() {
 
 jmp_buf CloseEnv;
 int IgnoreXIO(Display *d) {
-    cerr << APPNAME << ": connection to X server lost." << endl;
+    logStream << APPNAME << ": connection to X server lost." << endl;
     longjmp(CloseEnv, 1);
 }
 
 
 void App::StopServer() {
-    // Stop alars clock and ignore signals
-    alarm(0);
     signal(SIGQUIT, SIG_IGN);
     signal(SIGINT, SIG_IGN);
     signal(SIGHUP, SIG_IGN);
     signal(SIGPIPE, SIG_IGN);
     signal(SIGTERM, SIG_DFL);
     signal(SIGKILL, SIG_DFL);
-    signal(SIGALRM, SIG_DFL);
 
     // Catch X error
     XSetIOErrorHandler(IgnoreXIO);
@@ -980,7 +1011,7 @@ void App::StopServer() {
     // Send HUP to process group
     errno = 0;
     if((killpg(getpid(), SIGHUP) != 0) && (errno != ESRCH))
-        cerr << APPNAME << ": can't send HUP to process group " << getpid() << endl;
+        logStream << APPNAME << ": can't send HUP to process group " << getpid() << endl;
 
     // Send TERM to server
     if(ServerPID < 0)
@@ -988,7 +1019,7 @@ void App::StopServer() {
     errno = 0;
     if(killpg(ServerPID, SIGTERM) < 0) {
         if(errno == EPERM) {
-            cerr << APPNAME << ": can't kill X server" << endl;
+            logStream << APPNAME << ": can't kill X server" << endl;
             exit(ERR_EXIT);
         }
         if(errno == ESRCH)
@@ -997,11 +1028,11 @@ void App::StopServer() {
 
     // Wait for server to shut down
     if(!ServerTimeout(10, (char *)"X server to shut down")) {
-        cerr << endl;
+        logStream << endl;
         return;
     }
 
-    cerr << endl << APPNAME << ":  X server slow to shut down, sending KILL signal." << endl;
+    logStream << endl << APPNAME << ":  X server slow to shut down, sending KILL signal." << endl;
 
     // Send KILL to server
     errno = 0;
@@ -1012,10 +1043,10 @@ void App::StopServer() {
 
     // Wait for server to die
     if(ServerTimeout(3, (char*)"server to die")) {
-        cerr << endl << APPNAME << ": can't kill server" << endl;
+        logStream << endl << APPNAME << ": can't kill server" << endl;
         exit(ERR_EXIT);
     }
-    cerr << endl;
+    logStream << endl;
 }
 
 
@@ -1074,7 +1105,7 @@ void App::GetLock() {
         // no lockfile present, create one
         std::ofstream lockfile(cfg->getOption("lockfile").c_str(), ios_base::out);
         if (!lockfile) {
-            cerr << APPNAME << ": Could not create lock file: " << cfg->getOption("lockfile").c_str() << std::endl;
+            logStream << APPNAME << ": Could not create lock file: " << cfg->getOption("lockfile").c_str() << std::endl;
             exit(ERR_EXIT);
         }
         lockfile << getpid() << std::endl;
@@ -1088,13 +1119,13 @@ void App::GetLock() {
             // see if process with this pid exists
             int ret = kill(pid, 0);
             if (ret == 0 || (ret == -1 && errno == EPERM) ) {
-                cerr << APPNAME << ": Another instance of the program is already running with PID " << pid << std::endl;
+                logStream << APPNAME << ": Another instance of the program is already running with PID " << pid << std::endl;
                 exit(0);
             } else {
-                cerr << APPNAME << ": Stale lockfile found, removing it" << std::endl;
+                logStream << APPNAME << ": Stale lockfile found, removing it" << std::endl;
                 std::ofstream lockfile(cfg->getOption("lockfile").c_str(), ios_base::out);
                 if (!lockfile) {
-                    cerr << APPNAME << ": Could not create new lock file: " << cfg->getOption("lockfile") << std::endl;
+                    logStream << APPNAME << ": Could not create new lock file: " << cfg->getOption("lockfile") << std::endl;
                     exit(ERR_EXIT);
                 }
                 lockfile << getpid() << std::endl;
@@ -1109,26 +1140,30 @@ void App::RemoveLock() {
     remove(cfg->getOption("lockfile").c_str());
 }
 
+// Get server start check flag.
+bool App::isServerStarted() {
+    return serverStarted;
+}
+
 // Redirect stdout and stderr to log file
 void App::OpenLog() {
-    FILE *log = fopen (cfg->getOption("logfile").c_str(),"a");
-    if (!log) {
-        cerr <<  APPNAME << ": Could not accesss log file: " << cfg->getOption("logfile") << endl;
+
+    if ( !logStream.openLog( cfg->getOption("logfile").c_str() ) ) {
+        logStream <<  APPNAME << ": Could not accesss log file: " << cfg->getOption("logfile") << endl;
         RemoveLock();
         exit(ERR_EXIT);
     }
-    fclose(log);
-    freopen (cfg->getOption("logfile").c_str(),"a",stdout);
-    setvbuf(stdout, NULL, _IOLBF, BUFSIZ);
-    freopen (cfg->getOption("logfile").c_str(),"a",stderr);
-    setvbuf(stderr, NULL, _IONBF, BUFSIZ);
+    //
+    // I should set the buffers to imediate write, but I just flush on every
+    // << operation.
+    //
+    //
 }
 
-
 // Relases stdout/err
 void App::CloseLog(){
-    fclose(stderr);
-    fclose(stdout);
+	// Simply closing the log
+	logStream.closeLog();
 }
 
 string App::findValidRandomTheme(const string& set)
@@ -1153,7 +1188,7 @@ string App::findValidRandomTheme(const string& set)
         themefile = string(THEMESDIR) +"/" + name + THEMESFILE;
         if (stat(themefile.c_str(), &buf) != 0) {
             themes.erase(find(themes.begin(), themes.end(), name));
-            cerr << APPNAME << ": Invalid theme in config: "
+            logStream << APPNAME << ": Invalid theme in config: "
                  << name << endl;
             name = "";
         }
@@ -1213,7 +1248,7 @@ char* App::StrConcat(const char* str1, const char* str2) {
 void App::UpdatePid() {
     std::ofstream lockfile(cfg->getOption("lockfile").c_str(), ios_base::out);
     if (!lockfile) {
-        cerr << APPNAME << ": Could not update lock file: " << cfg->getOption("lockfile").c_str() << std::endl;
+        logStream << APPNAME << ": Could not update lock file: " << cfg->getOption("lockfile").c_str() << std::endl;
         exit(ERR_EXIT);
     }
     lockfile << getpid() << std::endl;
