@@ -32,14 +32,19 @@
 #define OK_EXIT     0
 #define ERR_EXIT    1
 
-// duration for showing error messages, as "login command failed", in seconds
+/* duration for showing error messages,
+ * as "login command failed", in seconds 
+ */
 #define ERROR_DURATION  5
 
 // variables replaced in login_cmd
 #define SESSION_VAR     "%session"
 #define THEME_VAR       "%theme"
 
-// variables replaced in pre-session_cmd and post-session_cmd
+/* variables replaced in pre-session_cmd and post-session_cmd */
 #define USER_VAR       "%user"
 
+/* max height/width for images */
+#define MAX_DIMENSION 10000
+
 #endif
